#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdio.h>
#include "image.h"
#define STREAM_ORIGINAL_FILE "stream_original"

#define TET_OK return 0
#define TET_FAIL return 1
#define TET_ABORT return -1
int main()
{
    g_type_init();

    GdkPixdata pixdata = {0,};
#ifdef PIXDATA_STRUCT
    memcpy(&pixdata, &my_image, sizeof(GdkPixdata));
#endif
#ifdef PIXDATA_MACROS
    pixdata.magic = GDK_PIXBUF_MAGIC_NUMBER;
    pixdata.height = MY_IMAGE_HEIGHT;
    pixdata.width = MY_IMAGE_WIDTH;
    pixdata.rowstride = MY_IMAGE_ROWSTRIDE;
#ifdef PIXDATA_RLE
    guint32 encoding = GDK_PIXDATA_ENCODING_RLE;
    pixdata.pixel_data = MY_IMAGE_RLE_PIXEL_DATA;
    pixdata.length = 0;
    //calculate length of stream(use mean of rle)
    unsigned char count;
    while((count = pixdata.pixel_data[pixdata.length]) != '\0')
    {
        pixdata.length += 1 + ((count & 0x80) ? MY_IMAGE_BYTES_PER_PIXEL : 1); 
    }
    pixdata.length += GDK_PIXDATA_HEADER_LENGTH;
#else /*define PIXDATA_RAW*/
    guint32 encoding = GDK_PIXDATA_ENCODING_RAW;
    pixdata.pixel_data = MY_IMAGE_RAW_PIXEL_DATA;
    pixdata.length = GDK_PIXDATA_HEADER_LENGTH + pixdata.rowstride * pixdata.height; 
#endif /*#ifdef PIXDATA_RLE*/
    guint32 colortype = MY_IMAGE_BYTES_PER_PIXEL == 4 ?
            GDK_PIXDATA_COLOR_TYPE_RGBA : GDK_PIXDATA_COLOR_TYPE_RGB; 
    guint32 sample_width = GDK_PIXDATA_SAMPLE_WIDTH_8;
    pixdata.pixdata_type = colortype | sample_width | encoding;
#endif /*#ifdef PIXDATA_RLE*/
#ifdef PIXDATA_STREAM
    if(!gdk_pixdata_deserialize(&pixdata, sizeof(my_image), my_image, NULL))
    {
        printf("Cannot read stream.\n");
        TET_FAIL;
    }
#endif /*#ifdef PIXDATA_STREAM*/
    // Read original pixdata from file containing a stream    
    GError *error = NULL;
    GIOChannel *channel = g_io_channel_new_file(STREAM_ORIGINAL_FILE, "r", &error);
    if(channel == NULL)
    {
        printf("Cannot open file \"%s\"for reading original data:\n%s.\n",
            STREAM_ORIGINAL_FILE, error->message);
        TET_ABORT;
    }
    if(g_io_channel_set_encoding(channel, NULL, &error) != G_IO_STATUS_NORMAL)
    {
        printf("Cannot read file as binary: %s.\n", error->message);
        TET_ABORT;
    }
    gsize length_orig = 0;
    guint8* stream_orig = NULL;
    g_io_channel_read_to_end(channel, &stream_orig, &length_orig, NULL);
    
    GdkPixdata pixdata_orig = {0,};
    if(!gdk_pixdata_deserialize(&pixdata_orig, (guint)length_orig, stream_orig, &error))
    {
        printf("Cannot read stream of original data:\n%s\n", error->message);
        TET_ABORT;
    }
    if(memcmp(&pixdata, &pixdata_orig, GDK_PIXDATA_HEADER_LENGTH) != 0)
    {
        printf("Header of resulting pixdata differs from the original data.\n");
        TET_FAIL;
    }
    if(memcmp(pixdata.pixel_data, pixdata_orig.pixel_data,
        pixdata.length - GDK_PIXDATA_HEADER_LENGTH) != 0)
    {
        printf("Pixels content of resulting pixdata differ from the original data.\n");
        TET_FAIL;
    }
    TET_OK;
}