#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdio.h>
#include "image.h"

#ifdef PIXDATA_DECODER

#ifndef MY_IMAGE_RUN_LENGTH_DECODE
#error MY_IMAGE_RUN_LENGTH_DECODE macro isn't defined
#endif

#define TET_OK return 0
#define TET_FAIL return 1
#define TET_ABORT return -1

const gchar* filenames[] = {"green_center.ico", "black_full.ico"};
int main()
{
    g_type_init();
    GdkPixbuf* pixbuf = NULL;
    GdkPixdata pixdata_raw = {0,};
    GdkPixdata pixdata_rle = {0,};
    guint8* result_pixels = NULL;
    gint i;
    for(i = 0; i < sizeof(filenames) / sizeof(filenames[0]); i++)
    {
        pixbuf = gdk_pixbuf_new_from_file(filenames[i], NULL);
        if(pixbuf == NULL)
        {
             printf("Cannot load image from file %s.\n", filenames[i]);
             TET_ABORT;
        }
        gdk_pixdata_from_pixbuf(&pixdata_raw, pixbuf, FALSE);

        result_pixels = g_new(guint8, pixdata_raw.length - GDK_PIXDATA_HEADER_LENGTH);
        if(result_pixels == NULL)
        {
            printf("Cannot allocate memory for buffer.\n");
            g_object_unref(pixbuf);
            TET_ABORT;
        }
        gdk_pixdata_from_pixbuf(&pixdata_rle, pixbuf, TRUE);
        guint32 colortype = pixdata_raw.pixdata_type & GDK_PIXDATA_COLOR_TYPE_MASK;
        
        MY_IMAGE_RUN_LENGTH_DECODE( 
            result_pixels                                       /*image_buf*/,
            pixdata_rle.pixel_data                              /*rle_data*/,
            pixdata_rle.length - GDK_PIXDATA_HEADER_LENGTH      /*size*/,
            (colortype ==  GDK_PIXDATA_COLOR_TYPE_RGBA) ? 4 : 3 /*bpp*/
            );
            
        if(memcmp(result_pixels, pixdata_raw.pixel_data, 
            pixdata_rle.length - GDK_PIXDATA_HEADER_LENGTH) != 0)
        {
            printf("Decoded pixels don't coincide with expected ones.\n");
            TET_FAIL;
        }
        g_free(pixdata_rle.pixel_data);
        g_free(result_pixels);
        g_object_unref(pixbuf);
    }
    TET_OK;
}
#else /*!define PIXDATA_DECODER*/

#ifdef MY_IMAGE_RUN_LENGTH_DECODE
#error MY_IMAGE_RUN_LENGTH_DECODE macro is defined
#endif /*#ifdef MY_IMAGE_RUN_LENGTH_DECODE*/

int main()
{
    return 0;
}
#endif /*#ifdef PIXBUF_DECODER*/
