#!/bin/sh

# Installs or uninstalls (if -u option is specified) custom pixbuf loader
# Place this script to the directory where the loader (checkboard.so) is.

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
cd ${WORK_DIR}

PATH=/opt/gnome/bin:$PATH

if [ -z ${GDK_PIXBUF_MODULE_FILE} ]
then
    if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]
    then
        SYSCONFDIR=/etc
        if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]  
        then
            SYSCONFDIR=/etc/opt/gnome
            if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]  
            then
                echo "Unable to locate directory containing the .loaders file."
                exit 1
            fi
        fi
    fi

    LFILES=`find ${SYSCONFDIR}/gtk-2.0/ -name '*.loaders*'`
    LFILES_NONL=`echo ${LFILES} | tr -d \n`

    if [ -z "${LFILES_NONL}" ]; then
        echo "Unable to locate the .loaders file in ${SYSCONFDIR}/gtk-2.0/ or its subdirectories."
        exit 1
    fi

    which arch >/dev/null 2>&1 
    if [ $? -eq 0 ]; then
        # arch command is available
        ARCHNAME=`arch`
        GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | grep ${ARCHNAME}`
    else
        # arch command is not available
        GDK_PIXBUF_MODULE_FILE=""
    fi
    
    if [ -z "${GDK_PIXBUF_MODULE_FILE}" ]; then
    # .loaders file has been found but not in a directory that has
    # architecture string in its name
        GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | tail -1`
    fi
fi

echo "Updating the list of loaders in ${GDK_PIXBUF_MODULE_FILE}"

QL_CMD=gdk-pixbuf-query-loaders-64
which ${QL_CMD} >/dev/null 2>&1 
if [ $? -ne 0 ] 
then
    QL_CMD=gdk-pixbuf-query-loaders
    
    which ${QL_CMD} >/dev/null 2>&1
    if [ $? -ne 0 ] 
    then
        QL_CMD=gdk-pixbuf-query-loaders-32
    fi
fi 

if [ "$1" = '-u' ] 
then
#uninstall
    echo "Uninstalling custom pixbuf loader..."
    ${QL_CMD} > ${GDK_PIXBUF_MODULE_FILE}
else
#install
    echo "Installing custom pixbuf loader..."
    ${QL_CMD} checkboard.so >> ${GDK_PIXBUF_MODULE_FILE} 
fi
